import { LRParser } from '@lezer/lr';
import { LRLanguage, indentNodeProp, delimitedIndent, foldNodeProp, foldInside, LanguageSupport } from '@codemirror/language';
import { styleTags, tags } from '@lezer/highlight';
import { snippetCompletion, completeFromList } from '@codemirror/autocomplete';

// This file was generated by lezer-generator. You probably shouldn't edit it.
const parser = LRParser.deserialize({
  version: 14,
  states: "!WQYQPOOOkQPO'#CeOOQO'#Cj'#CjOOQO'#Cf'#CfQYQPOOOOQO,59P,59POrQPO,59POOQO-E6d-E6dOOQO1G.k1G.k",
  stateData: "|~O]OSPOS~ORQOSQOTQOUQOWPO~OVTO~PYOVWO~PYOST~",
  goto: "v_PPPPPPPPP`fPPPpXQOPSUQSOQUPTVSUXROPSU",
  nodeNames: "⚠ LineComment Program Identifier String Name Number ) ( Application",
  maxTerm: 14,
  nodeProps: [
    ["openedBy", 7,"("],
    ["closedBy", 8,")"]
  ],
  skippedNodes: [0,1],
  repeatNodeCount: 1,
  tokenData: "%X~R_XY!QYZ!Q]^!Qpq!Qst!cxy!zyz#P}!O#U!O!P#g!Q![#[!c!v#u!v!w#}!w!}#u#R#S$q#T#o$q~!VS]~XY!QYZ!Q]^!Qpq!Q~!hSP~OY!cZ;'S!c;'S;=`!t<%lO!c~!wP;=`<%l!c~#POW~~#UOV~~#XP!Q![#[~#aQU~!O!P#g!Q![#[~#jP!Q![#m~#rPU~!Q![#m~#zPT~#T#o#u~$SQT~!Q![$Y#T#o#u~$]Q!O!P$c!Q![$Y~$fP!Q![$i~$nPS~!Q![$i~$vRR~!Q![%P#R#S$q#T#o$q~%UPR~!Q![%P",
  tokenizers: [0],
  topRules: {"Program":[0,2]},
  tokenPrec: 41
});

/// A collection of JavaScript-related
/// [snippets](#autocomplete.snippet).
const snippets = [
    // compare
    snippetCompletion("(${value1} if ${condition} else ${value2})", {
        label: "if ... else",
        type: "keyword"
    }),
    // math
    snippetCompletion("dif(${tag})", {
        label: "dif",
        type: "keyword"
    }),
    snippetCompletion("max(${tag})", {
        label: "max",
        type: "keyword"
    }),
    snippetCompletion("min(${tag})", {
        label: "min",
        type: "keyword"
    }),
    snippetCompletion("avg(${tag})", {
        label: "avg",
        type: "keyword"
    }),
    snippetCompletion("abs(${param})", {
        label: "abs",
        type: "keyword"
    }),
    snippetCompletion("exp(${param})", {
        label: "exp",
        type: "keyword"
    }),
    snippetCompletion("log(${param})", {
        label: "log",
        type: "keyword"
    }),
    snippetCompletion("log10(${param})", {
        label: "log10",
        type: "keyword"
    }),
    snippetCompletion("sqrt(${param})", {
        label: "sqrt",
        type: "keyword"
    }),
    snippetCompletion("pow(${param}, ${degree})", {
        label: "pow",
        type: "keyword"
    }),
    snippetCompletion("hypot(${param}, ${param2})", {
        label: "hypot",
        type: "keyword"
    }),
    snippetCompletion("round(${param})", {
        label: "round",
        type: "keyword"
    }),
    snippetCompletion("floor(${param})", {
        label: "floor",
        type: "keyword"
    }),
    snippetCompletion("ceil(${param})", {
        label: "ceil",
        type: "keyword"
    }),
    // constants 
    snippetCompletion("Pi", {
        label: "Pi",
        type: "constant"
    }),
    snippetCompletion("E", {
        label: "E",
        type: "constant"
    }),
    // trigonometric
    snippetCompletion("sin(${param})", {
        label: "sin",
        type: "keyword"
    }),
    snippetCompletion("asin(${param})", {
        label: "asin",
        type: "keyword"
    }),
    snippetCompletion("sinh(${param})", {
        label: "sinh",
        type: "keyword"
    }),
    snippetCompletion("cos(${param})", {
        label: "cos",
        type: "keyword"
    }),
    snippetCompletion("acos(${param})", {
        label: "acos",
        type: "keyword"
    }),
    snippetCompletion("cosh(${param})", {
        label: "cosh",
        type: "keyword"
    }),
    snippetCompletion("tan(${param})", {
        label: "tan",
        type: "keyword"
    }),
    snippetCompletion("atan(${param})", {
        label: "atan",
        type: "keyword"
    }),
    snippetCompletion("tanh(${param})", {
        label: "tanh",
        type: "keyword"
    }),
];

const mathLanguage = LRLanguage.define({
    name: 'math',
    parser: parser.configure({
        props: [
            indentNodeProp.add({
                Application: delimitedIndent({ closing: ")", align: false })
            }),
            foldNodeProp.add({
                Application: foldInside
            }),
            styleTags({
                Identifier: tags.variableName,
                Name: tags.name,
                String: tags.string,
                Number: tags.number,
                LineComment: tags.lineComment,
                "( )": tags.paren
            })
        ]
    }),
    languageData: {
        commentTokens: { line: ";" }
    }
});
function math() {
    return new LanguageSupport(mathLanguage, [
        mathLanguage.data.of({
            autocomplete: completeFromList(snippets)
        })
    ]);
}

export { math, mathLanguage };
